/****************************************************************************
** Meta object code from reading C++ file 'customcolorscheme.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../mauikit-terminal-4.0.2/src/lib/customcolorscheme.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'customcolorscheme.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17CustomColorSchemeE_t {};
} // unnamed namespace

template <> constexpr inline auto CustomColorScheme::qt_create_metaobjectdata<qt_meta_tag_ZN17CustomColorSchemeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CustomColorScheme",
        "descriptionChanged",
        "",
        "description",
        "backgroundColorChanged",
        "QColor",
        "backgroundColor",
        "foregroundColorChanged",
        "foregroundColor",
        "color2Changed",
        "color2",
        "color3Changed",
        "color3",
        "color4Changed",
        "color4",
        "color5Changed",
        "color5",
        "color6Changed",
        "color6",
        "color7Changed",
        "color7",
        "color8Changed",
        "color8",
        "color9Changed",
        "color9",
        "setDescription",
        "setBackgroundColor",
        "setForegroundColor",
        "setColor2",
        "setColor3",
        "setColor4",
        "setColor5",
        "setColor6",
        "setColor7",
        "setColor8",
        "setColor9",
        "name"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'descriptionChanged'
        QtMocHelpers::SignalData<void(QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'backgroundColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'foregroundColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 8 },
        }}),
        // Signal 'color2Changed'
        QtMocHelpers::SignalData<void(QColor)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 10 },
        }}),
        // Signal 'color3Changed'
        QtMocHelpers::SignalData<void(QColor)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 12 },
        }}),
        // Signal 'color4Changed'
        QtMocHelpers::SignalData<void(QColor)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 14 },
        }}),
        // Signal 'color5Changed'
        QtMocHelpers::SignalData<void(QColor)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 16 },
        }}),
        // Signal 'color6Changed'
        QtMocHelpers::SignalData<void(QColor)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 18 },
        }}),
        // Signal 'color7Changed'
        QtMocHelpers::SignalData<void(QColor)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 20 },
        }}),
        // Signal 'color8Changed'
        QtMocHelpers::SignalData<void(QColor)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 22 },
        }}),
        // Signal 'color9Changed'
        QtMocHelpers::SignalData<void(QColor)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 24 },
        }}),
        // Slot 'setDescription'
        QtMocHelpers::SlotData<void(QString)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'setBackgroundColor'
        QtMocHelpers::SlotData<void(QColor)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'setForegroundColor'
        QtMocHelpers::SlotData<void(QColor)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 8 },
        }}),
        // Slot 'setColor2'
        QtMocHelpers::SlotData<void(QColor)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 10 },
        }}),
        // Slot 'setColor3'
        QtMocHelpers::SlotData<void(QColor)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 12 },
        }}),
        // Slot 'setColor4'
        QtMocHelpers::SlotData<void(QColor)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 14 },
        }}),
        // Slot 'setColor5'
        QtMocHelpers::SlotData<void(QColor)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 16 },
        }}),
        // Slot 'setColor6'
        QtMocHelpers::SlotData<void(QColor)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 18 },
        }}),
        // Slot 'setColor7'
        QtMocHelpers::SlotData<void(QColor)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 20 },
        }}),
        // Slot 'setColor8'
        QtMocHelpers::SlotData<void(QColor)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 22 },
        }}),
        // Slot 'setColor9'
        QtMocHelpers::SlotData<void(QColor)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'name'
        QtMocHelpers::PropertyData<QString>(36, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
        // property 'description'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'backgroundColor'
        QtMocHelpers::PropertyData<QColor>(6, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'foregroundColor'
        QtMocHelpers::PropertyData<QColor>(8, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'color2'
        QtMocHelpers::PropertyData<QColor>(10, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'color3'
        QtMocHelpers::PropertyData<QColor>(12, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'color4'
        QtMocHelpers::PropertyData<QColor>(14, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'color5'
        QtMocHelpers::PropertyData<QColor>(16, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'color6'
        QtMocHelpers::PropertyData<QColor>(18, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'color7'
        QtMocHelpers::PropertyData<QColor>(20, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8),
        // property 'color8'
        QtMocHelpers::PropertyData<QColor>(22, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 9),
        // property 'color9'
        QtMocHelpers::PropertyData<QColor>(24, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 10),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CustomColorScheme, qt_meta_tag_ZN17CustomColorSchemeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CustomColorScheme::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17CustomColorSchemeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17CustomColorSchemeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17CustomColorSchemeE_t>.metaTypes,
    nullptr
} };

void CustomColorScheme::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CustomColorScheme *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->descriptionChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->backgroundColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 2: _t->foregroundColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 3: _t->color2Changed((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 4: _t->color3Changed((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 5: _t->color4Changed((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 6: _t->color5Changed((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 7: _t->color6Changed((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 8: _t->color7Changed((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 9: _t->color8Changed((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 10: _t->color9Changed((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 11: _t->setDescription((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->setBackgroundColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 13: _t->setForegroundColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 14: _t->setColor2((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 15: _t->setColor3((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 16: _t->setColor4((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 17: _t->setColor5((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 18: _t->setColor6((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 19: _t->setColor7((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 20: _t->setColor8((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 21: _t->setColor9((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CustomColorScheme::*)(QString )>(_a, &CustomColorScheme::descriptionChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CustomColorScheme::*)(QColor )>(_a, &CustomColorScheme::backgroundColorChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CustomColorScheme::*)(QColor )>(_a, &CustomColorScheme::foregroundColorChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CustomColorScheme::*)(QColor )>(_a, &CustomColorScheme::color2Changed, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (CustomColorScheme::*)(QColor )>(_a, &CustomColorScheme::color3Changed, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (CustomColorScheme::*)(QColor )>(_a, &CustomColorScheme::color4Changed, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (CustomColorScheme::*)(QColor )>(_a, &CustomColorScheme::color5Changed, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (CustomColorScheme::*)(QColor )>(_a, &CustomColorScheme::color6Changed, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (CustomColorScheme::*)(QColor )>(_a, &CustomColorScheme::color7Changed, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (CustomColorScheme::*)(QColor )>(_a, &CustomColorScheme::color8Changed, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (CustomColorScheme::*)(QColor )>(_a, &CustomColorScheme::color9Changed, 10))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->description(); break;
        case 2: *reinterpret_cast<QColor*>(_v) = _t->backgroundColor(); break;
        case 3: *reinterpret_cast<QColor*>(_v) = _t->foregroundColor(); break;
        case 4: *reinterpret_cast<QColor*>(_v) = _t->color2(); break;
        case 5: *reinterpret_cast<QColor*>(_v) = _t->color3(); break;
        case 6: *reinterpret_cast<QColor*>(_v) = _t->color4(); break;
        case 7: *reinterpret_cast<QColor*>(_v) = _t->color5(); break;
        case 8: *reinterpret_cast<QColor*>(_v) = _t->color6(); break;
        case 9: *reinterpret_cast<QColor*>(_v) = _t->color7(); break;
        case 10: *reinterpret_cast<QColor*>(_v) = _t->color8(); break;
        case 11: *reinterpret_cast<QColor*>(_v) = _t->color9(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setDescription(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setBackgroundColor(*reinterpret_cast<QColor*>(_v)); break;
        case 3: _t->setForegroundColor(*reinterpret_cast<QColor*>(_v)); break;
        case 4: _t->setColor2(*reinterpret_cast<QColor*>(_v)); break;
        case 5: _t->setColor3(*reinterpret_cast<QColor*>(_v)); break;
        case 6: _t->setColor4(*reinterpret_cast<QColor*>(_v)); break;
        case 7: _t->setColor5(*reinterpret_cast<QColor*>(_v)); break;
        case 8: _t->setColor6(*reinterpret_cast<QColor*>(_v)); break;
        case 9: _t->setColor7(*reinterpret_cast<QColor*>(_v)); break;
        case 10: _t->setColor8(*reinterpret_cast<QColor*>(_v)); break;
        case 11: _t->setColor9(*reinterpret_cast<QColor*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *CustomColorScheme::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CustomColorScheme::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17CustomColorSchemeE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int CustomColorScheme::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void CustomColorScheme::descriptionChanged(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void CustomColorScheme::backgroundColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void CustomColorScheme::foregroundColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void CustomColorScheme::color2Changed(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void CustomColorScheme::color3Changed(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void CustomColorScheme::color4Changed(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void CustomColorScheme::color5Changed(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void CustomColorScheme::color6Changed(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void CustomColorScheme::color7Changed(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void CustomColorScheme::color8Changed(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void CustomColorScheme::color9Changed(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
