/****************************************************************************
** Meta object code from reading C++ file 'ksession.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../mauikit-terminal-4.0.2/src/lib/ksession.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ksession.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8KSessionE_t {};
} // unnamed namespace

template <> constexpr inline auto KSession::qt_create_metaobjectdata<qt_meta_tag_ZN8KSessionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KSession",
        "started",
        "",
        "finished",
        "copyAvailable",
        "termGetFocus",
        "termLostFocus",
        "termKeyPressed",
        "QKeyEvent*",
        "changedKeyBindings",
        "kb",
        "titleChanged",
        "historySizeChanged",
        "initialWorkingDirectoryChanged",
        "matchFound",
        "startColumn",
        "startLine",
        "endColumn",
        "endLine",
        "noMatchFound",
        "hasActiveProcessChanged",
        "foregroundProcessNameChanged",
        "processHasSilent",
        "value",
        "bellRequest",
        "message",
        "monitorSilenceChanged",
        "currentDirChanged",
        "shellProgramChanged",
        "argsChanged",
        "setKeyBindings",
        "setTitle",
        "name",
        "startShellProgram",
        "sendSignal",
        "signal",
        "setShellProgram",
        "progname",
        "shellProgram",
        "setArgs",
        "args",
        "getShellPID",
        "changeDir",
        "dir",
        "sendText",
        "text",
        "sendKey",
        "rep",
        "key",
        "mod",
        "clearScreen",
        "search",
        "regexp",
        "forwards",
        "selectionChanged",
        "textSelected",
        "getForegroundProcessError",
        "sessionFinished",
        "createSession",
        "std::unique_ptr<Session>",
        "kbScheme",
        "initialWorkingDirectory",
        "title",
        "shellProgramArgs",
        "history",
        "hasActiveProcess",
        "foregroundProcessName",
        "currentDir",
        "historySize",
        "monitorSilence"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'started'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'copyAvailable'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'termGetFocus'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'termLostFocus'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'termKeyPressed'
        QtMocHelpers::SignalData<void(QKeyEvent *, bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 }, { QMetaType::Bool, 2 },
        }}),
        // Signal 'changedKeyBindings'
        QtMocHelpers::SignalData<void(QString)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Signal 'titleChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'historySizeChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'initialWorkingDirectoryChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'matchFound'
        QtMocHelpers::SignalData<void(int, int, int, int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { QMetaType::Int, 16 }, { QMetaType::Int, 17 }, { QMetaType::Int, 18 },
        }}),
        // Signal 'noMatchFound'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasActiveProcessChanged'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'foregroundProcessNameChanged'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'processHasSilent'
        QtMocHelpers::SignalData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Signal 'bellRequest'
        QtMocHelpers::SignalData<void(QString)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Signal 'monitorSilenceChanged'
        QtMocHelpers::SignalData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentDirChanged'
        QtMocHelpers::SignalData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shellProgramChanged'
        QtMocHelpers::SignalData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'argsChanged'
        QtMocHelpers::SignalData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setKeyBindings'
        QtMocHelpers::SlotData<void(const QString &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'setTitle'
        QtMocHelpers::SlotData<void(QString)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 32 },
        }}),
        // Slot 'startShellProgram'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sendSignal'
        QtMocHelpers::SlotData<bool(int)>(34, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 35 },
        }}),
        // Slot 'setShellProgram'
        QtMocHelpers::SlotData<void(const QString &)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 37 },
        }}),
        // Slot 'shellProgram'
        QtMocHelpers::SlotData<QString() const>(38, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setArgs'
        QtMocHelpers::SlotData<void(const QStringList &)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 40 },
        }}),
        // Slot 'args'
        QtMocHelpers::SlotData<QStringList() const>(40, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'getShellPID'
        QtMocHelpers::SlotData<int()>(41, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'changeDir'
        QtMocHelpers::SlotData<void(const QString &)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 43 },
        }}),
        // Slot 'sendText'
        QtMocHelpers::SlotData<void(QString)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 45 },
        }}),
        // Slot 'sendKey'
        QtMocHelpers::SlotData<void(int, int, int) const>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 47 }, { QMetaType::Int, 48 }, { QMetaType::Int, 49 },
        }}),
        // Slot 'clearScreen'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'search'
        QtMocHelpers::SlotData<void(const QString &, int, int, bool)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 52 }, { QMetaType::Int, 16 }, { QMetaType::Int, 15 }, { QMetaType::Bool, 53 },
        }}),
        // Slot 'search'
        QtMocHelpers::SlotData<void(const QString &, int, int)>(51, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 52 }, { QMetaType::Int, 16 }, { QMetaType::Int, 15 },
        }}),
        // Slot 'search'
        QtMocHelpers::SlotData<void(const QString &, int)>(51, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 52 }, { QMetaType::Int, 16 },
        }}),
        // Slot 'search'
        QtMocHelpers::SlotData<void(const QString &)>(51, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 52 },
        }}),
        // Slot 'selectionChanged'
        QtMocHelpers::SlotData<void(bool)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 55 },
        }}),
        // Slot 'getForegroundProcessError'
        QtMocHelpers::SlotData<int() const>(56, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'sessionFinished'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'createSession'
        QtMocHelpers::SlotData<std::unique_ptr<Session>(QString)>(58, 2, QMC::AccessPrivate, 0x80000000 | 59, {{
            { QMetaType::QString, 32 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'kbScheme'
        QtMocHelpers::PropertyData<QString>(60, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable, 6),
        // property 'initialWorkingDirectory'
        QtMocHelpers::PropertyData<QString>(61, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(62, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'shellProgram'
        QtMocHelpers::PropertyData<QString>(38, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 18),
        // property 'shellProgramArgs'
        QtMocHelpers::PropertyData<QStringList>(63, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable, 19),
        // property 'history'
        QtMocHelpers::PropertyData<QString>(64, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'hasActiveProcess'
        QtMocHelpers::PropertyData<bool>(65, QMetaType::Bool, QMC::DefaultPropertyFlags, 12),
        // property 'foregroundProcessName'
        QtMocHelpers::PropertyData<QString>(66, QMetaType::QString, QMC::DefaultPropertyFlags, 13),
        // property 'currentDir'
        QtMocHelpers::PropertyData<QString>(67, QMetaType::QString, QMC::DefaultPropertyFlags, 17),
        // property 'historySize'
        QtMocHelpers::PropertyData<int>(68, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'monitorSilence'
        QtMocHelpers::PropertyData<bool>(69, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KSession, qt_meta_tag_ZN8KSessionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KSession::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8KSessionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8KSessionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8KSessionE_t>.metaTypes,
    nullptr
} };

void KSession::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KSession *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->started(); break;
        case 1: _t->finished(); break;
        case 2: _t->copyAvailable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->termGetFocus(); break;
        case 4: _t->termLostFocus(); break;
        case 5: _t->termKeyPressed((*reinterpret_cast<std::add_pointer_t<QKeyEvent*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 6: _t->changedKeyBindings((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->titleChanged(); break;
        case 8: _t->historySizeChanged(); break;
        case 9: _t->initialWorkingDirectoryChanged(); break;
        case 10: _t->matchFound((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 11: _t->noMatchFound(); break;
        case 12: _t->hasActiveProcessChanged(); break;
        case 13: _t->foregroundProcessNameChanged(); break;
        case 14: _t->processHasSilent((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->bellRequest((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->monitorSilenceChanged(); break;
        case 17: _t->currentDirChanged(); break;
        case 18: _t->shellProgramChanged(); break;
        case 19: _t->argsChanged(); break;
        case 20: _t->setKeyBindings((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->setTitle((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->startShellProgram(); break;
        case 23: { bool _r = _t->sendSignal((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: _t->setShellProgram((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 25: { QString _r = _t->shellProgram();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 26: _t->setArgs((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 27: { QStringList _r = _t->args();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 28: { int _r = _t->getShellPID();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 29: _t->changeDir((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 30: _t->sendText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 31: _t->sendKey((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 32: _t->clearScreen(); break;
        case 33: _t->search((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 34: _t->search((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 35: _t->search((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 36: _t->search((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 37: _t->selectionChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: { int _r = _t->getForegroundProcessError();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 39: _t->sessionFinished(); break;
        case 40: { std::unique_ptr<Session> _r = _t->createSession((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<std::unique_ptr<Session>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KSession::*)()>(_a, &KSession::started, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)()>(_a, &KSession::finished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)(bool )>(_a, &KSession::copyAvailable, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)()>(_a, &KSession::termGetFocus, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)()>(_a, &KSession::termLostFocus, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)(QKeyEvent * , bool )>(_a, &KSession::termKeyPressed, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)(QString )>(_a, &KSession::changedKeyBindings, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)()>(_a, &KSession::titleChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)()>(_a, &KSession::historySizeChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)()>(_a, &KSession::initialWorkingDirectoryChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)(int , int , int , int )>(_a, &KSession::matchFound, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)()>(_a, &KSession::noMatchFound, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)()>(_a, &KSession::hasActiveProcessChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)()>(_a, &KSession::foregroundProcessNameChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)(bool )>(_a, &KSession::processHasSilent, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)(QString )>(_a, &KSession::bellRequest, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)()>(_a, &KSession::monitorSilenceChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)()>(_a, &KSession::currentDirChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)()>(_a, &KSession::shellProgramChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (KSession::*)()>(_a, &KSession::argsChanged, 19))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->getKeyBindings(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->getInitialWorkingDirectory(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->getTitle(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->shellProgram(); break;
        case 4: *reinterpret_cast<QStringList*>(_v) = _t->args(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->getHistory(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->hasActiveProcess(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->foregroundProcessName(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->currentDir(); break;
        case 9: *reinterpret_cast<int*>(_v) = _t->historySize(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->monitorSilence(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setKeyBindings(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setInitialWorkingDirectory(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setTitle(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setShellProgram(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setArgs(*reinterpret_cast<QStringList*>(_v)); break;
        case 9: _t->setHistorySize(*reinterpret_cast<int*>(_v)); break;
        case 10: _t->setMonitorSilence(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KSession::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KSession::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8KSessionE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KSession::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 41)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 41;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 41)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 41;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void KSession::started()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KSession::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KSession::copyAvailable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KSession::termGetFocus()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KSession::termLostFocus()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KSession::termKeyPressed(QKeyEvent * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void KSession::changedKeyBindings(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void KSession::titleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void KSession::historySizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void KSession::initialWorkingDirectoryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void KSession::matchFound(int _t1, int _t2, int _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 11
void KSession::noMatchFound()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void KSession::hasActiveProcessChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void KSession::foregroundProcessNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void KSession::processHasSilent(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void KSession::bellRequest(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void KSession::monitorSilenceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void KSession::currentDirChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void KSession::shellProgramChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void KSession::argsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}
QT_WARNING_POP
